// Alarm_Clock_I2C_Mini_Terminal Joe Pardue August 14, 2013
// This is the Arduino side software for several early 2013 Nuts&Volts 
// articles. You may find a version of these articles at  
// www.smileymicros.com click on the Articles and Downloads button
//
// This is an educational demonstration and is neither the best way to
// do things nor does it have more than cursory error catching.
//
// Most of the DS1307 timekeeping items were taken from RTCLib provided by
// Jean-Claude Wippler of www.jeelabs.com
//
// Date and time functions using a DS1307 RTC connected via I2C and Wire lib
//
// I organized this code in four modules:
// 1. Alarm_Clock - this module - headers, variables, defines, setup(), loop(), and timer interrupt.
// 2. Commander - parses commands from the PC.
// 3. Date_Time - functions for using dates and times
// 4. Alarm_Clock - alarm service functions.

#include <I2CMiniTerminal.h>
#include <Wire.h> // Arduino I2C library 

#include <TimerOne.h> // Timer 1 library
#include <EEPROM.h>
#include "RTClib.h" // DS1307 Real Time Clock library
#include <dht22.h> // Library from Virtuabotix

#define DEBUG // Comment out for production cod

//
// Button debounce stuff from Arduino IDE example
//
// constants won't change. They're used here to 
// set pin numbers:
const int buttonPin = 10; // the number of the pushbutton pin
const int ledPin =  13;   // the number of the LED driver pin
const int speaker = 9;   // the number of the speaker driver pin
// Variables will change:
int ledState = HIGH;         // the current state of the output pin
int buttonState;             // the current reading from the input pin
int lastButtonState = LOW;   // the previous reading from the input pin
// The following variables are long's because the time, measured in miliseconds,
// will quickly become a bigger number than can be stored in an int.
long lastDebounceTime = 0;  // the last time the output pin was toggled
long debounceDelay = 50;    // the debounce time; increase if the output flickers

// defines for the EEPROM addresses
#define A1A 0
#define A2A 6
#define A3A 13
#define A4A 20
#define A5A 27

struct alarm {
  DateTime datetime;
  uint8_t type;
  uint8_t is_set;
  uint8_t is_tripped;
};

alarm alarm1;
alarm alarm2;
alarm alarm3;
alarm alarm4;
alarm alarm5;
alarm alarm6; // not an alarm, used to test calibration

RTC_DS1307 RTC; // Used by RTClib

// Communication Constants
#define TERMINATOR '!' // Terminator character
#define SEPARATOR ',' // Separator character
#define MAX_COMMAND_LENGTH 16
#define MAX_COMMANDS 8
#define MAX_MESSAGE MAX_COMMAND_LENGTH * MAX_COMMANDS
#define MAX_NUMBER_SIZE 6

// Communication arrays
byte commandArray[MAX_COMMAND_LENGTH];
byte messageArray[MAX_MESSAGE];
// Communication counts
int commandArrayCount = 0;
int messageArrayCount = 0;

// A timer is used to set the check_alarm flag  
volatile int check_alarm = 0;
volatile int check_alarm_count = 0;

// TEMP - removed arrays to get more RAM for test
// Piezo tone variables
int three_short_beep = 0;
//int three_short[] = { 1, 0, 1, 0, 1, 0, 2};
int three_long_beep = 0;
//int three_long[] = { 1, 1, 0, 1, 1, 0, 1, 1, 0, 2};
int sos_beep = 0;
//int sos[] = { 1, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 0, 2};
int warble3_beep = 0;
//int warble3[] = { 1, 2, 0, 1, 2, 0, 1, 2, 0, 3};
int beep_count = 0;

// Calibration variables
uint32_t calSeconds; // value sent from PC

//uint32_t adjustSeconds; 

//uint32_t cal_remaining; // decremented each second
uint32_t lastSet; // the unixtime for the last time the PC set the RTC
uint8_t addSubtract; // add or subtract a second each cal_seconds
#define CAL_EEPROM_ADDRESS 34 // follows alarms
// Couldn't this be replaced by having last_cal = 0 if not set?
uint8_t calIsSet; // has the calibratio been set?

// For rtcInterrupt
//volatile int sqwCount = 0; 

p_string(progname) = "IMTclock";
p_string(progrev) = "Rev 001";

// 0 for civilian time, 1 for military time
uint8_t militaryTime = 0;

void setup()
{
  
  Wire.begin(); 
      
  // Identify code on LCD    
  mt_clear();
  mt_printPM(progname,0,0);
  mt_printPM(progrev,0,1);
  delay(2000);
  
  // Set up the serial port
  Serial.begin(57600);
  
  // Say howdy
  Serial.println(F("Alarm_Clock_I2C_Mini_Terminal rev. 0.01")); 
  
  EEPROM.write(A1A+5,255);
  EEPROM.write(A2A+5,255);
  EEPROM.write(A3A+5,255);
  EEPROM.write(A4A+5,255);
  EEPROM.write(A5A+5,255);
  
 /* 
  Serial.println("BEFORE");
  Serial.print("alarm1.is_set = ");
  Serial.println(alarm1.is_set);
  Serial.print("alarm2.is_set = ");
  Serial.println(alarm2.is_set);
  Serial.print("alarm3.is_set = ");
  Serial.println(alarm3.is_set);
  Serial.print("alarm4.is_set = ");
  Serial.println(alarm4.is_set);
  Serial.print("alarm5.is_set = ");
  Serial.println(alarm5.is_set); 
 */
  initializeMessageArray(); // Initialize message array
  initializeDS1307RTC(); // Initialize the external RTC
  initializeCalibration(); // Initialize the calibration values
  //initializeAlarms(); // Initailize the alarms

  // initialize timer1
  Timer1.initialize(250000); // trips 4 times a second
  Timer1.attachInterrupt(myTimer1); 

#if defined(DEBUG)  
  if(!RTC.isrunning()) Serial.println(F("RTC is NOT running."));  
  else Serial.println(F("RTC is running.")); 
#endif 

/*
  Serial.println("AFTER");
  Serial.print("alarm1.is_set = ");
  Serial.println(alarm1.is_set);
  Serial.print("alarm2.is_set = ");
  Serial.println(alarm2.is_set);
  Serial.print("alarm3.is_set = ");
  Serial.println(alarm3.is_set);
  Serial.print("alarm4.is_set = ");
  Serial.println(alarm4.is_set);
  Serial.print("alarm5.is_set = ");
  Serial.println(alarm5.is_set);
  
  delay(10000);*/

/*   mt_clear(); // Clear the LCD 
  
  mt_printPM(progname,0,0);
  mt_printPM(progrev,0,1); 
  //delay(20000); // So you can view the progname and progrev
 
  // Start clear and zero'd cursor
  mt_clear();
  mt_cursor();
  mt_setCursor(0,0); */
  
  //DateTime now = RTC.now();
  //delay(250);
 // mt_clear();
  mt_print("Howdy",0,0);
  mt_print("Ya'll",0,1);
  delay(1000);
 
}

void showDateTime()
{
#if defined(DEBUG)   
  Serial.println(F("showDateTime")); 
#endif    
  // show date and time on LCD 
  char buff[9];
  
  DateTime myDateTime = RTC.now();
  
  dateToChar(buff,myDateTime);
  mt_print(buff,0,0);
  Serial.print(F("Date :"));
  Serial.println(buff);
  timeToChar(buff, myDateTime);
  Serial.print(F("Time: "));
  Serial.println(buff);  
  mt_print(buff,0,1);
} 
   


void loop()
{ 
  showDateTime();
  delay(1000);
}

// Rather than use the loop to check the alarm continuously
// and thereby adding quite a load on the processor, we will
// use a timer set to check the alarms once per second
void myTimer1()
{ 
  // Timer ticks at 250 ms, 
  // so 4 ticks per second to check alarm
  if(check_alarm_count++ >= 3){
    check_alarm = 1;
    check_alarm_count = 0;
  }

 /* if(three_short_beep){    
     if(three_short[beep_count] == 0) noTone(speaker);
     else if(three_short[beep_count] == 1) tone(speaker,4300);
     else if(three_short[beep_count] == 2){
       three_short_beep = 0;
       beep_count = -1;
     }
     beep_count++;
  }
  if(three_long_beep){
     if(three_long[beep_count] == 0) noTone(speaker);
     else if(three_long[beep_count] == 1) tone(speaker,4300);
     else if(three_long[beep_count] == 2){
       three_long_beep = 0;
       beep_count = -1;
     }
     beep_count++;
  }  
  if(sos_beep){
     if(sos[beep_count] == 0) noTone(speaker);
     else if(sos[beep_count] == 1) tone(speaker,4300);
     else if(sos[beep_count] == 2){
       sos_beep = 0;
       beep_count = -1;
     }
     beep_count++;
  }       
  if(warble3_beep){
     if(warble3[beep_count] == 0) noTone(speaker);
     else if(warble3[beep_count] == 1) tone(speaker,4000);
     else if(warble3[beep_count] == 2) tone(speaker,4300);
     else if(beep_count >= 3){
       warble3_beep = 0;
       beep_count = -1;
     }
     beep_count++;
  }*/ 
}



