//
// Commander module for the command processing functions
//
// This software demonstrates some principles in receiving text
// from a PC and interpreting that text as commands and numbers.

// Clear out the message array in preparation for receiving
// a new message.
void initializeMessageArray()
{
#if defined(DEBUG)   
  //Serial.println(F("initializeMessageArray")); 
#endif   
   // Fill with the terminator character
   for(int i = 0; i < MAX_MESSAGE; i++)
  {    
    // Fill array with the terminal character
    messageArray[i] = TERMINATOR;
  }   
}

// Load a message from the PC until receiving the message 
// terminator symbol, then call the message parsing function.
void loadMessageArray()
{
#if defined(DEBUG)   
  //Serial.println(F("loadMessageArray")); 
#endif 
  if(messageArrayCount <= MAX_MESSAGE)
  { 
    // put the character in the array
    messageArray[messageArrayCount] = Serial.read();
  }
  
  if(messageArray[messageArrayCount] == TERMINATOR)
  {   
    parseArray();  
  }
  else messageArrayCount++;
}

// Parse the message array for a valid message. If a valid 
// message is present, then parse the message as a command.
void parseArray()
{    
  int i = 0;
  int j = 0;
  while( (messageArray[i] != TERMINATOR) && (i < MAX_MESSAGE) )
  {
    while( (messageArray[i] != SEPARATOR) && (messageArray[i] != TERMINATOR))
    {
      commandArray[j++] = messageArray[i++]; 
    } 
    commandArrayCount = j;
    parseCommand();
    j = 0;
    i++;
  }
  messageArrayCount=0;
  initializeMessageArray(); 

  // Clear the receive buffer before going back to the loop()
  while(Serial.available()) Serial.read(); 
}

// Parse a command from the PC and call the function
// associated with the command
void parseCommand()
{
  switch (commandArray[0])
  {
     case 'A': // Set Alarm # 1 to 5
       ACase();
       break;  
      case 'c': // calbrate
       calibrate();
       break;  
      case 'C': // read CR or clear CL calibrate
       CCase();
       break;        
      case 'D': // Date
       DCase();
       break; 
      case 'G': // Get
       GCase();
       break;          
      case 'N': // show NOW
       NCase();
       break;      
      case 'P': // Use PC to set date time
       PCase();
       break; 
      case 'T': // Get the RTC time
       TCase();
       break;
      case 't': // Tests for debugging
       tCase();
       break;       
      case 'U': // Unset alarm
       UCase();
       break;        
     default:
       Serial.print(F("parseCommand Bad commandArray[0] = "));
       Serial.println(commandArray[0]);
       //Serial.println(commandArray[1]);
       //Serial.println(commandArray[2]);
       //Serial.println(commandArray[3]);
       //Serial.println(commandArray[4]);
       break;  
  } 
}

//
// Functions associated with cases
//

void tCase(){
#if defined(DEBUG)   
  Serial.println(F("tCase"));
#endif
 
  Serial.print(F("Before auto adjustment:"));
  //calSeconds – from EEPROM
  Serial.print(F("calSeconds = "));
  Serial.println(calSeconds);
  //addSubtract – from EEPROM
  Serial.print(F("addSubtract = "));
  Serial.println(addSubtract);  
  //lastSal – from EEPROM
  Serial.print(F("lastSet = "));
  Serial.println(lastSet);  
  //rtcTime – read from RTC
  DateTime now = RTC.now();
  Serial.print(F("rtcTime = "));
  Serial.println(now.unixtime()); 
   
  //rtcTimeSinceLastCal - calculated
  //if(now.unixtime() > lastSet){
    //Serial.print(F("rtcTimeSinceLastCal = "));
    //Serial.println(now.unixtime() - lastCal);
    //}
  //else {
    //Serial.println(F("ERROR in tCase: now.unixtime() NOT > lastCal"));
  //}
  //adjustmentSeconds - calculated
  Serial.print(F("adjustmentSeconds = "));
  Serial.println((now.unixtime() - lastSet)/calSeconds);  
  //adjustedTime - calculated
  //Serial.print(F("adjustedTime = "));
  //Serial.println(0);
  
 // Adjust calSeconds
 // elapsed time = (pc time) - (lastSet)
 // error time = (pc time) - (rtc time)
 // calSeconds = (elapsed time) / (error time)

  // get time from pc
  int i;
  uint32_t pcUnixTime = 0;
  uint32_t mult = 1000000000;
  
 Serial.print(F("characters for unixtime: ")); 
  for(i=1;i<=10;i++){
  Serial.print(commandArray[i]-48,DEC);
   pcUnixTime += (commandArray[i]-48) * mult;
   mult = mult/10;
   if(commandArray[i] == '-')break;
  }
  Serial.println("");
  Serial.print(F("PC time: "));
  Serial.println(pcUnixTime);
  Serial.print(F("now.unixtime(): "));
  Serial.println(now.unixtime());
  Serial.print(F("((pcUnixTime - now.unixtime()): "));
  Serial.println((pcUnixTime - now.unixtime())); 
  Serial.print(F(" (pcUnixTime - lastSet)"));
  Serial.println((pcUnixTime - lastSet));
 
    Serial.print(F("RTC time: "));
    Serial.print(now.hour(), DEC);
    Serial.print(':');
    Serial.print(now.minute(), DEC);
    Serial.print(':');
    Serial.println(now.second(), DEC);
 
  calSeconds = (pcUnixTime - lastSet) / (pcUnixTime - now.unixtime());  
  Serial.print(F("calSeconds: "));
  Serial.println(calSeconds);
}


// Set Alarm # 1 to 9
void ACase(){
#if defined(DEBUG)   
  Serial.println(F("ACase"));
#endif 
  switch (commandArray[1])
  {
     case '1': // Alarm1
       setAlarm(alarm1.datetime);
       alarm1.is_set = 1;
       alarm1.is_tripped = 0;
       alarm1.type = commandArray[14]-48;
       showDate("A1", alarm1.datetime);
       writeAlarm(1,alarm1);
       break; 
     case '2': // Alarm2
       setAlarm(alarm2.datetime);
       alarm2.is_set = 1;
       alarm2.is_tripped = 0;
       alarm2.type = commandArray[14]-48;
       showDate("A2", alarm2.datetime);
       writeAlarm(2,alarm2);
       break;
      case '3': // Alarm3
       setAlarm(alarm3.datetime);
       alarm3.is_set = 1;
       alarm3.is_tripped = 0;
       alarm3.type = commandArray[14]-48;
       showDate("A3", alarm3.datetime);
       writeAlarm(3,alarm3);
       break;     
      case '4': // Alarm4
       setAlarm(alarm4.datetime);
       alarm4.is_set = 1;
       alarm5.is_tripped = 0;
       alarm4.type = commandArray[14]-48;
       showDate("A4", alarm4.datetime);
       writeAlarm(4,alarm4);
       break;      
      case '5': // Alarm
       setAlarm(alarm5.datetime);
       alarm5.is_set = 1;
       alarm5.is_tripped = 0;
       alarm5.type = commandArray[14]-48;
       showDate("A5", alarm5.datetime);
       writeAlarm(5,alarm5);
       break;  
      case '6': // USED TO TEST CALIBRATION
       setAlarm(alarm6.datetime);
       alarm6.is_set = 1;
       alarm6.is_tripped = 0;
       alarm6.type = commandArray[14]-48;
       showDate("A6", alarm6.datetime);
       //Set RTC to the indicated future value
       setToFuture();      
       break;       
     default:
       Serial.println(F("ACase Bad commandArray[1]."));
       break;   
  }  
}

void CCase(){
#if defined(DEBUG)   
  Serial.println(F("CCase"));
#endif   
  switch (commandArray[1])
  {
     case 'C':
       clearCalibration();
       break; 
     case 'R':
       Serial.println(F("CR Case"));
       readCalibration();
       break;        
     default:
       Serial.print(F("CCase Bad commandArray[1] = "));
       Serial.println(commandArray[1]);
       break; 
  }  
}

 // Date
void DCase(){
#if defined(DEBUG)   
  Serial.println(F("DCase")); 
#endif 

  if( (commandArray[1] == 'a') && \
    (commandArray[2] == 't') &&     \
    (commandArray[3] == 'e') ){
      showNowDate();
      showNowAdjustedDate();
    }
  else  Serial.println(F("DCase Bad commandArray[]."));  
}

// Get
void GCase(){
#if defined(DEBUG)   
  //Serial.println(F("GCase")); 
#endif   
  switch (commandArray[1])
  {
     case 'A': // Get alarm data
       GACase();
       break; 
     /*case 'C':
       GCCase();
       break; */     
     default:
       Serial.print(F("GCase Bad commandArray[1] = "));
       Serial.println(commandArray[1]);
       break; 
  }
}

// Get Alarm
void GACase(){
#if defined(DEBUG)   
  //Serial.println(F("GACase")); 
#endif 
  uint8_t ut1,ut2,ut3,ut4;
  
  switch (commandArray[2])
  {
     case '1': 
       //Serial.println(str);
       if(alarm1.is_set){
         ut1 = (uint8_t)(alarm1.datetime.unixtime() & 0x000000FF);
         ut2 = (uint8_t)((alarm1.datetime.unixtime() & 0x0000FF00)>>8);
         ut3 = (uint8_t)((alarm1.datetime.unixtime() & 0x00FF0000)>>16);
         ut4 = (uint8_t)((alarm1.datetime.unixtime() & 0xFF000000)>>24);     
         Serial.print(F("GA1"));         
         Serial.print(SEPARATOR);
         Serial.print(ut1);
         Serial.print(SEPARATOR);
         Serial.print(ut2);
         Serial.print(SEPARATOR);
         Serial.print(ut3);
         Serial.print(SEPARATOR);
         Serial.print(ut4);
         Serial.print(SEPARATOR);
         Serial.print(alarm1.type);
         Serial.print(SEPARATOR);
         Serial.print(alarm1.is_set);
         Serial.print(SEPARATOR);
         Serial.print(alarm1.is_tripped);
         Serial.println(TERMINATOR);
       }
       else 
       {
         Serial.println(F("GA0!")); // Alarm not set
       }
       break; 
     case '2': 
       if(alarm2.is_set){
         ut1 = (uint8_t)(alarm2.datetime.unixtime() & 0x000000FF);
         ut2 = (uint8_t)((alarm2.datetime.unixtime() & 0x0000FF00)>>8);
         ut3 = (uint8_t)((alarm2.datetime.unixtime() & 0x00FF0000)>>16);
         ut4 = (uint8_t)((alarm2.datetime.unixtime() & 0xFF000000)>>24);     
         Serial.print(F("GA2"));         
         Serial.print(SEPARATOR);
         Serial.print(ut1);
         Serial.print(SEPARATOR);
         Serial.print(ut2);
         Serial.print(SEPARATOR);
         Serial.print(ut3);
         Serial.print(SEPARATOR);
         Serial.print(ut4);
         Serial.print(SEPARATOR);
         Serial.print(alarm2.type);
         Serial.print(SEPARATOR);
         Serial.print(alarm2.is_set);
         Serial.print(SEPARATOR);
         Serial.print(alarm2.is_tripped);
         Serial.println(TERMINATOR);
       }
       else 
       {
         Serial.println(F("GA0!")); // Alarm not set
       }
       break;
     case '3': 
       if(alarm3.is_set){
         ut1 = (uint8_t)(alarm3.datetime.unixtime() & 0x000000FF);
         ut2 = (uint8_t)((alarm3.datetime.unixtime() & 0x0000FF00)>>8);
         ut3 = (uint8_t)((alarm3.datetime.unixtime() & 0x00FF0000)>>16);
         ut4 = (uint8_t)((alarm3.datetime.unixtime() & 0xFF000000)>>24);     
         Serial.print(F("GA3"));         
         Serial.print(SEPARATOR);
         Serial.print(ut1);
         Serial.print(SEPARATOR);
         Serial.print(ut2);
         Serial.print(SEPARATOR);
         Serial.print(ut3);
         Serial.print(SEPARATOR);
         Serial.print(ut4);
         Serial.print(SEPARATOR);
         Serial.print(alarm3.type);
         Serial.print(SEPARATOR);
         Serial.print(alarm3.is_set);
         Serial.print(SEPARATOR);
         Serial.print(alarm3.is_tripped);
         Serial.println(TERMINATOR);
       }
       else 
       {
         Serial.println(F("GA0!")); // Alarm not set
       }
       break;  
     case '4': 
       if(alarm4.is_set){
         ut1 = (uint8_t)(alarm4.datetime.unixtime() & 0x000000FF);
         ut2 = (uint8_t)((alarm4.datetime.unixtime() & 0x0000FF00)>>8);
         ut3 = (uint8_t)((alarm4.datetime.unixtime() & 0x00FF0000)>>16);
         ut4 = (uint8_t)((alarm4.datetime.unixtime() & 0xFF000000)>>24);     
         Serial.print(F("GA4"));         
         Serial.print(SEPARATOR);
         Serial.print(ut1);
         Serial.print(SEPARATOR);
         Serial.print(ut2);
         Serial.print(SEPARATOR);
         Serial.print(ut3);
         Serial.print(SEPARATOR);
         Serial.print(ut4);
         Serial.print(SEPARATOR);
         Serial.print(alarm4.type);
         Serial.print(SEPARATOR);
         Serial.print(alarm4.is_set);
         Serial.print(SEPARATOR);
         Serial.print(alarm4.is_tripped);
         Serial.println(TERMINATOR);
       }
       else 
       {
         Serial.println(F("GA0!")); // Alarm not set
       }
       break;  
     case '5': 
       if(alarm5.is_set)
       {
         ut1 = (uint8_t)(alarm5.datetime.unixtime() & 0x000000FF);
         ut2 = (uint8_t)((alarm5.datetime.unixtime() & 0x0000FF00)>>8);
         ut3 = (uint8_t)((alarm5.datetime.unixtime() & 0x00FF0000)>>16);
         ut4 = (uint8_t)((alarm5.datetime.unixtime() & 0xFF000000)>>24);     
         Serial.print(F("GA5"));         
         Serial.print(SEPARATOR);
         Serial.print(ut1);
         Serial.print(SEPARATOR);
         Serial.print(ut2);
         Serial.print(SEPARATOR);
         Serial.print(ut3);
         Serial.print(SEPARATOR);
         Serial.print(ut4);
         Serial.print(SEPARATOR);
         Serial.print(alarm5.type);
         Serial.print(SEPARATOR);
         Serial.print(alarm5.is_set);
         Serial.print(SEPARATOR);
         Serial.print(alarm5.is_tripped);
         Serial.println(TERMINATOR);
       }
       else 
       {
         Serial.println(F("GA0!")); // Alarm not set
       }
       break;   
     default:
       Serial.print(F("GACase Bad commandArray[1] = "));
       Serial.println(commandArray[2]);
       break; 
  }
  // PC AAC requests all 5 alarms when it loads. This chokes
  // the Arduino, so a 1/4 second delay give it time to think
  delay(250);
}

/* // Is the RTC running
void ICase(){
#if defined(DEBUG)   
  Serial.println(F("ICase"); 
#endif 

  if(!RTC.isrunning()) Serial.println("N");  
  else Serial.println("Y"); 
}
*/

// Show now
void NCase(){
#if defined(DEBUG)
  Serial.println(F("NCase")); 
#endif
  showNow();
}

// Set RTC to PC time
void PCase(){
#if defined(DEBUG)
  Serial.println(F("PCase"));
#endif 
  setDateTimeToPC();
}

// Show the time
void TCase(){
#if defined(DEBUG)
  //Serial.println(F("TCase")); 
#endif 
  if( (commandArray[1] == 'i') && \
    (commandArray[2] == 'm') && \
    (commandArray[3] == 'e') ){
      showTime();
      showAdjustedTime(); 
    }
  else  Serial.println("TCase Bad commandArray[].");  
}

// Unset an alarm
void UCase(){
#if defined(DEBUG)
 Serial.print(F("UCase Unset Alarm"));
 Serial.println(commandArray[1]);
#endif 
  if( (commandArray[1] < '6') && (commandArray[1]> '0') ) unsetAlarm(commandArray[1]);
  else Serial.println("Error in UCase - alarm number out of range");
}

