//
// Date_Time module for processing date and time related functions
//

// Initialzie the I2C communications and the DS1307 Real Time Clock
void initializeDS1307RTC(){
#if defined(DEBUG)   
  Serial.println(F("initializeDS1307RTC")); 
#endif   
  Wire.begin(); // initialize the I2C communications
  RTC.begin(); // initialize the DS1307 real time clock
}

// 
// Convert DateTime to characters for 8x2 LCD
// For digital times < 10, we are used to seeing the hour
// with a space in front and the minute and second with
// a 0 in front.
//
void timeToChar(char * tim, DateTime myDateTime)
{
  
  char buff[4]; 
  
  int hour = myDateTime.hour();
  
  // The raw data is in 24 hour time - military - format
  // We set militaryTime to 1 for militaryTime and to 0
  // for civilian time. to show Civilian time we subtract
  // 12 if it the 24 hour time is greater than 12
  if( !militaryTime && (hour >= 12)) hour -= 12;  
  
  // convert the hour
  itoa(hour,buff,10);
  if(hour >= 10)
  {
    tim[0] = buff[0];
    tim[1] = buff[1];
  }
  else
  {
    tim[0] = ' ';
    tim[1] = buff[0];    
  }
  tim[2] = ':';
  // Convert the minute
  int min = myDateTime.minute();
  itoa(min,buff,10);
  if(min >= 10)
  {
    tim[3] = buff[0];
    tim[4] = buff[1];
  }
  else
  {
    tim[3] = '0';
    tim[4] = buff[0];    
  }
  tim[5] = ':';  
  // Convert the second
  int sec = myDateTime.second();
  itoa(sec,buff,10); 
  if(sec >= 10)
  {
    tim[6] = buff[0];
    tim[7] = buff[1];
  }
  else
  {
    tim[6] = '0';
    tim[7] = buff[0];    
  }

  // make the char array a string by terminating
  // it with the NULL '\0' character
  tim[8] = '\0';   
}

void dateToChar(char * tim, DateTime myDateTime)
{
  char buff[4]; 
  
  int month = myDateTime.month();
  itoa(month,buff,10);
  
  if(month >= 10)
  {
    tim[0] = buff[0];
    tim[1] = buff[1];
  }
  else
  {
    tim[0] = '0';
    tim[1] = buff[0];    
  }
  tim[2] = '/';
 
  int day = myDateTime.day();
  itoa(day,buff,10);
  if(day >= 10)
  {
    tim[3] = buff[0];
    tim[4] = buff[1];
  }
  else
  {
    tim[3] = '0';
    tim[4] = buff[0];    
  }
  tim[5] = '/';  
 
  int year = myDateTime.year();
  itoa(year,buff,10); 
  if(year >= 10)
  {
    tim[6] = buff[2];
    tim[7] = buff[3];
  }
  else
  {
    tim[6] = '0';
    tim[7] = buff[2];    
  }

  tim[8] = '\0';   
}

// This is used by the PC AAC to retrieve the time
void showTime()
{
    DateTime now = RTC.now();

    Serial.print(F("TIME"));
    Serial.print(now.hour(), DEC);
    Serial.print(':');
    Serial.print(now.minute(), DEC);
    Serial.print(':');
    Serial.print(now.second(), DEC);
    Serial.println(TERMINATOR);  

}


//
// Show dates and times
// NOTE: most of this made sense during debugging
// but can now be consolidated into fewer functions;
// TODO: DO THAT
//
void showDate(const char* txt, const DateTime& dt) {
#if defined(DEBUG)   
  Serial.println(F("showDate")); 
#endif  
    Serial.print(txt);
    Serial.print(' ');
    Serial.print(dt.year(), DEC);
    Serial.print('/');
    Serial.print(dt.month(), DEC);
    Serial.print('/');
    Serial.print(dt.day(), DEC);
    Serial.print(' ');
    Serial.print(dt.hour(), DEC);
    Serial.print(':');
    Serial.print(dt.minute(), DEC);
    Serial.print(':');
    Serial.print(dt.second(), DEC);
    Serial.println(TERMINATOR);
}

// Show the current date and time
void showNow()
{
#if defined(DEBUG)   
  Serial.println(F("showNow")); 
#endif   
    DateTime now = RTC.now();
    
    Serial.print(now.year(), DEC);
    Serial.print('/');
    Serial.print(now.month(), DEC);
    Serial.print('/');
    Serial.print(now.day(), DEC);
    Serial.print(' ');
    Serial.print(now.hour(), DEC);
    Serial.print(':');
    Serial.print(now.minute(), DEC);
    Serial.print(':');
    Serial.print(now.second(), DEC); 
    Serial.println(TERMINATOR);    
}

// Show the current date
void showNowDate()
{
#if defined(DEBUG)   
  Serial.println(F("showNowDate")); 
#endif   
    DateTime now = RTC.now();
    
    Serial.print(F("DATE"));
    Serial.print(now.year(), DEC);
    Serial.print('/');
    Serial.print(now.month(), DEC);
    Serial.print('/');
    Serial.print(now.day(), DEC);
    Serial.print(' ');
    Serial.println(TERMINATOR);    
}
// Show the current adjusted date
void showNowAdjustedDate()
{
#if defined(DEBUG)   
  Serial.println(F("showNowADate")); 
#endif   
    DateTime now = RTC.now();
    uint32_t adjustedTime, adjustSeconds;

    adjustSeconds = (now.unixtime() - lastSet)/calSeconds; 
    
    if(addSubtract) adjustedTime = now.unixtime() - adjustSeconds;
    else adjustedTime = now.unixtime() + adjustSeconds;    
    
    DateTime temp(adjustedTime);  
        
    Serial.print(F("ADAT"));
    Serial.print(temp.year(), DEC);
    Serial.print('/');
    Serial.print(temp.month(), DEC);
    Serial.print('/');
    Serial.print(temp.day(), DEC);
    Serial.print(' ');
    Serial.println(TERMINATOR);    
}

void showAdjustedTime()
{
#if defined(DEBUG)   
  //Serial.print(F("showAdjustedTime));// adjustSeconds = "));
  //Serial.println(adjustSeconds); 
#endif  
  if(calIsSet){
    DateTime now = RTC.now();
    uint32_t adjustedTime, adjustSeconds;

    adjustSeconds = (now.unixtime() - lastSet)/calSeconds; 
    if(addSubtract) adjustedTime = now.unixtime() - adjustSeconds;
    else adjustedTime = now.unixtime() + adjustSeconds;

    DateTime temp(adjustedTime);

    Serial.print(F("ATIM"));
    Serial.print(temp.hour(), DEC);
    Serial.print(':');
    Serial.print(temp.minute(), DEC);
    Serial.print(':');
    Serial.print(temp.second(), DEC);
    Serial.println(TERMINATOR);  
  } 
  else Serial.println("Error: showAdjustedTime - calIsSet is 0");  
}

// Get the date and time from the PC and set the 
// date and time variables
void setDateTimeToPC(){  
#if defined(DEBUG) 
 Serial.println(F("setDateTimeToPC"));
#endif

 int i;
 uint32_t myUnixTime = 0;
 uint32_t mult = 1000000000;
 
 //Serial.print(F("characters for unixtime: "));
 for(i=1;i<=10;i++){
  //Serial.print(commandArray[i]-48,DEC);
   myUnixTime += (commandArray[i]-48) * mult;
   mult = mult/10;
   if(commandArray[i] == '-')break;
 }
 
 RTC.adjust(myUnixTime);
 
 // Set lastSet to now
 lastSet = myUnixTime;
 writeLastSet();
 
#if defined(DEBUG)
  Serial.println(F("After adjusting RTC, showNow(): "));
  showNow();
#endif
}

//
// Calibration functions
//
void initializeCalibration(){
#if defined(DEBUG) 
 Serial.println(F("initializeCalibration"));
#endif   

  readCalibration();
}

void writeCalibration(){
#if defined(DEBUG)   
  Serial.println(F("writeCalibration")); 
#endif   
  uint8_t val1,val2,val3,val4;

  val1 = (uint8_t)(calSeconds & 0x000000FF);
  val2 = (uint8_t)((calSeconds & 0x0000FF00)>>8); 
  val3 = (uint8_t)((calSeconds & 0x00FF0000)>>16); 
  val4 = (uint8_t)((calSeconds & 0xFF000000)>>24); 
  EEPROM.write(CAL_EEPROM_ADDRESS,val1);
  EEPROM.write(CAL_EEPROM_ADDRESS+1,val2);
  EEPROM.write(CAL_EEPROM_ADDRESS+2,val3);
  EEPROM.write(CAL_EEPROM_ADDRESS+3,val4);
  
  writeLastSet();
  
  EEPROM.write(CAL_EEPROM_ADDRESS+8,calIsSet);
  EEPROM.write(CAL_EEPROM_ADDRESS+9,addSubtract);        
}

void writeLastSet(){
  uint8_t val1,val2,val3,val4;
  
  val1 = (uint8_t)(lastSet & 0x000000FF);
  val2 = (uint8_t)((lastSet & 0x0000FF00)>>8); 
  val3 = (uint8_t)((lastSet & 0x00FF0000)>>16); 
  val4 = (uint8_t)((lastSet & 0xFF000000)>>24); 
  EEPROM.write(CAL_EEPROM_ADDRESS+4,val1);
  EEPROM.write(CAL_EEPROM_ADDRESS+5,val2);
  EEPROM.write(CAL_EEPROM_ADDRESS+6,val3);
  EEPROM.write(CAL_EEPROM_ADDRESS+7,val4);   
}

void readCalibration(){
#if defined(DEBUG)   
  Serial.println(F("readCalibration")); 
#endif   
  calSeconds = (uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS) + \
                ((uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+1)<<8) + \
                ((uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+2)<<16) + \
                ((uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+3)<<24);
  lastSet = (uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+4) + \
                ((uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+5)<<8) + \
                ((uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+6)<<16) + \
                ((uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+7)<<24);                              
                               
  calIsSet = EEPROM.read(CAL_EEPROM_ADDRESS+8);
  addSubtract = EEPROM.read(CAL_EEPROM_ADDRESS+9);
       
#if defined(DEBUG)   
  Serial.println(F("read_Calibration"));
  Serial.print(F("calSeconds = "));
  Serial.println(calSeconds);
  Serial.print(F("lastSet = "));
  Serial.println(lastSet); 
  Serial.print(F("calIsSet = "));
  Serial.println(calIsSet);
  Serial.print(F("addSubtract = "));
  Serial.println(addSubtract); 
  //Serial.print(F("cal_remaining = "));//not an EEPROM value
  //Serial.println(cal_remaining); 
#endif      
}

void clearCalibration(){
#if defined(DEBUG)   
  Serial.println(F("clearCalibration")); 
#endif   
  uint8_t val1,val2,val3,val4;

  EEPROM.write(CAL_EEPROM_ADDRESS,0);
  EEPROM.write(CAL_EEPROM_ADDRESS+1,0);
  EEPROM.write(CAL_EEPROM_ADDRESS+2,0);
  EEPROM.write(CAL_EEPROM_ADDRESS+3,0);
  EEPROM.write(CAL_EEPROM_ADDRESS+4,0);
  EEPROM.write(CAL_EEPROM_ADDRESS+5,0);
  EEPROM.write(CAL_EEPROM_ADDRESS+6,0);
  EEPROM.write(CAL_EEPROM_ADDRESS+7,0); 
  EEPROM.write(CAL_EEPROM_ADDRESS+8,0);
  EEPROM.write(CAL_EEPROM_ADDRESS+9,0);          
}

void calibrate(){
  #if defined(DEBUG)   
  Serial.println(F("calibrate()")); 
  #endif 
  uint8_t val1,val2,val3,val4;

  // Get the four bytes for the 32-bit Unix time
  val1 = commandArray[1];
  val2 = commandArray[2];
  val3 = commandArray[3];  
  val4 = commandArray[4];  
  
  // Get the add or subtract variable
  addSubtract = commandArray[5];  
  
  // Create the 32-bit calibration value from the 4 bytes
  calSeconds = (uint32_t)val1 + \
        ((uint32_t)(val2)<<8) + \
        ((uint32_t)(val3)<<16) + \
        ((uint32_t)(val4)<<24);

  // Set calibration to active
  calIsSet = 1; 
  
  // Use 0 to turn it off the calibration alarm
  if(calSeconds == 0){
    calIsSet = 0; // Calibration is no longer active
    Serial.println(F("Stop calibration"));
    return;
  }

  // Get the date and time from the DS1307
  DateTime now = RTC.now(); 
  // Load this into the initial last_cal variable
  lastSet = now.unixtime();
  
  // Save values to EEPROM
  writeCalibration();
  
  // Inform the PC  
  Serial.print(F("calSeconds = "));
  Serial.println(calSeconds);   
  Serial.print(F("addSubtract = "));
  Serial.println(addSubtract);
  Serial.print(F("lastSet = ")); 
  Serial.println(lastSet);
}


// Used to test the calibration by setting the RTC to
// some future date and see if the calibration seconds
// are correct
#if defined(DEBUG)
void setToFuture(){
#if defined(DEBUG)   
  Serial.println(F("setToFuture")); 
#endif  
  DateTime dt;
  
  dt.set_year(( (commandArray[2]-48) * 10) + (commandArray[3]-48)); 
  dt.set_month(( (commandArray[4]-48) * 10) + (commandArray[5]-48));
  dt.set_day(( (commandArray[6]-48) * 10) + (commandArray[7]-48)); 
  dt.set_hour(( (commandArray[8]-48) * 10) + (commandArray[9]-48)); 
  dt.set_minute(( (commandArray[10]-48) * 10) + (commandArray[11]-48)); 
  dt.set_second(( (commandArray[12]-48) * 10) + (commandArray[13]-48)); 
 
  uint32_t myUnixTime = 0;
  
  myUnixTime = dt.unixtime();
  RTC.adjust(myUnixTime);
  
  Serial.println(F("In setToFuture() after adjusting RTC, showNow(): "));
  showNow();   
}
#endif

