//
// Process_Alarms module for the time keeping functions
//
// Most of the DS1307 timekeeping items and concepts were taken from 
// RTCLib provided by Jean-Claude Wippler of www.jeelabs.com
//

void initializeAlarms(){ 
  
  for(int i = 1; i <= 5; i++){
    readAlarm(i);
  }
}

void goAlarm1(){
#if defined(DEBUG)   
  Serial.println(F("go_alarm1"));
#endif 
  if(alarm1.is_set){
    DateTime now = RTC.now();  
    if(now.unixtime() >= alarm1.datetime.unixtime()){
      #if defined(DEBUG)
      Serial.println(F("Alarm1 tripped!")); 
      #endif 
      alarm1.is_tripped = 1;      
      if(alarm1.type == 0){// If it is one shot, unset and clear EEPROM data
          unsetAlarm('1');    
      }     
      else if(alarm1.type == 1){// if daily, add 24 hrs to the alarm and leave set
        #if defined(DEBUG)
        Serial.print(F("Old alarm date time = "));
        showDate("alarm1",alarm1.datetime);
        #endif 
        // Add one day, 86400 seconds
        DateTime temp(alarm1.datetime.unixtime() + 86400L); 
        alarm1.datetime = temp;
        #if defined(DEBUG)
        Serial.print(F("New alarm date time = "));
        showDate("alarm1",alarm1.datetime);
        #endif
        // Write new alarm data to EEPROM
        writeAlarm(1,alarm1);          
      }// if alarm1 is daily 
    }// if now > alarm1 
    
    // Do what you wanted to do with this alarm if it is tripped
    if(alarm1.is_tripped){   
      // if the button state is not pressed, run the alarm
      if(!buttonState) warble3_beep = 1; 
      else {
        alarm1.is_tripped = 0; // turn this alarm off
        warble3_beep = 0; // turn the beep off  
      }     
    }// if alarm1.is_tripped
  }// if alarm1._is_set  
}

void goAlarm2(){
#if defined(DEBUG)   
  Serial.println(F("go_alarm2"));
#endif 
  if(alarm2.is_set){
    DateTime now = RTC.now();  
    if(now.unixtime() >= alarm2.datetime.unixtime()){
      #if defined(DEBUG)
      Serial.println(F("Alarm2 tripped!")); 
      #endif 
      alarm2.is_tripped = 1;      
      if(alarm2.type == 0){// If it is one shot, unset and clear EEPROM data
          unsetAlarm('2');    
      }     
      else if(alarm2.type == 1){// if daily, add 24 hrs to the alarm and leave set
        #if defined(DEBUG)
        Serial.print(F("Old alarm date time = "));
        showDate("alarm2",alarm2.datetime);
        #endif 
        // Add one day, 86400 seconds
        DateTime temp(alarm2.datetime.unixtime() + 86400L); 
        alarm2.datetime = temp;
        #if defined(DEBUG)
        Serial.print(F("New alarm date time = "));
        showDate("alarm2",alarm1.datetime);
        #endif
        // Write new alarm data to EEPROM
        writeAlarm(2,alarm1);          
      }// if alarm2 is daily 
    }// if now > alarm1 
    
    // Do what you wanted to do with this alarm if it is tripped
    if(alarm2.is_tripped){   
      // if the button state is not pressed, run the alarm
      if(!buttonState) warble3_beep = 1; 
      else {
        alarm2.is_tripped = 0; // turn this alarm off
        warble3_beep = 0; // turn the beep off  
      }     
    }// if alarm2.is_tripped
  }// if alarm2._is_set 

}

void goAlarm3(){
#if defined(DEBUG)   
  Serial.println(F("go_alarm3"));
#endif  
}

void goAlarm4(){
#if defined(DEBUG)   
  Serial.println(F("go_alarm4"));
#endif  
}

void goAlarm5(){
#if defined(DEBUG)   
  Serial.println(F("go_alarm5"));
#endif  
}

// Sets the Alarms datetime, but not the other alarm parameters
// Use: setAlarm(alarm1.datetime);
void setAlarm(DateTime& dt){
#if defined(DEBUG)
 Serial.println(F("setAlarm")); 
#endif 
  dt.set_year(( (commandArray[2]-48) * 10) + (commandArray[3]-48)); 
  dt.set_month(( (commandArray[4]-48) * 10) + (commandArray[5]-48));
  dt.set_day(( (commandArray[6]-48) * 10) + (commandArray[7]-48)); 
  dt.set_hour(( (commandArray[8]-48) * 10) + (commandArray[9]-48)); 
  dt.set_minute(( (commandArray[10]-48) * 10) + (commandArray[11]-48)); 
  dt.set_second(( (commandArray[12]-48) * 10) + (commandArray[13]-48)); 

}

//todo unset each alarm and record to eeprom
void unsetAlarm(uint8_t alarm_number){//DateTime& dt){
#if defined(DEBUG)
 Serial.println(F("unset_alarm")); 
#endif 
  // alarm_number is sent as character
  switch (alarm_number)
  {
     case '1': // write alarm 1
       Serial.print(F("unset alarm1"));
       Serial.println(TERMINATOR);
       EEPROM.write(A1A,0);
       EEPROM.write(A1A+1,0);
       EEPROM.write(A1A+2,0);
       EEPROM.write(A1A+3,0);
       EEPROM.write(A1A+4,0);
       EEPROM.write(A1A+5,0);
       EEPROM.write(A1A+6,0); 
       readAlarm(1); // load the variables
       break; 
     case '2': // write alarm 2
       Serial.println(F("unset alarm2"));
       Serial.println(TERMINATOR);
       EEPROM.write(A2A,0);
       EEPROM.write(A2A+1,0);
       EEPROM.write(A2A+2,0);
       EEPROM.write(A2A+3,0);
       EEPROM.write(A2A+4,0);
       EEPROM.write(A2A+5,0);
       EEPROM.write(A2A+6,0);      
       readAlarm(2); // load the variables
       break;  
     case '3': // write alarm 3
       Serial.println(F("unset alarm3"));
       Serial.println(TERMINATOR);
       EEPROM.write(A3A,0);
       EEPROM.write(A3A+1,0);
       EEPROM.write(A3A+2,0);
       EEPROM.write(A3A+3,0);
       EEPROM.write(A3A+4,0);
       EEPROM.write(A3A+5,0);
       EEPROM.write(A3A+6,0); 
       readAlarm(3); // load the variables
       break;
     case '4': // write alarm 4
       Serial.println(F("unset alarm4"));
       Serial.println(TERMINATOR);
       EEPROM.write(A4A,0);
       EEPROM.write(A4A+1,0);
       EEPROM.write(A4A+2,0);
       EEPROM.write(A4A+3,0);
       EEPROM.write(A4A+4,0);
       EEPROM.write(A4A+5,0);
       EEPROM.write(A4A+6,0); 
       readAlarm(4); // load the variables
       break;  
     case '5': // write alarm 5
       Serial.println(F("unset alarm5"));
       Serial.println(TERMINATOR);
       EEPROM.write(A5A,0);
       EEPROM.write(A5A+1,0);
       EEPROM.write(A5A+2,0);
       EEPROM.write(A5A+3,0);
       EEPROM.write(A5A+4,0);
       EEPROM.write(A5A+5,0);
       EEPROM.write(A5A+6,0); 
       readAlarm(5); // load the variables
       break;  
       default:
       Serial.print(F("ERROR: unset_alarm() bad alarm_number = "));
       Serial.println(alarm_number);
       break; 
  }       
}

// Note - this is educational novice friendly code and does not use
// some more effiecient, but also more difficult C constructs.
void writeAlarm(int alarm_number, struct alarm alar){
#if defined(DEBUG)
 Serial.println(F("writeAlarm")); 
#endif 
  uint8_t val1,val2,val3,val4;
  uint32_t unix_value = (uint32_t)alar.datetime.unixtime();
  val1 = (uint8_t)(unix_value & 0x000000FF);
  val2 = (uint8_t)((unix_value & 0x0000FF00)>>8); 
  val3 = (uint8_t)((unix_value & 0x00FF0000)>>16); 
  val4 = (uint8_t)((unix_value & 0xFF000000)>>24); 

#if defined(DEBUG)
  Serial.print(F("For alarm #: "));
  Serial.println(alarm_number);
  Serial.print(F("For datetime = "));
  Serial.println(unix_value);
  Serial.print(F("val1 = "));
  Serial.println(val1);
  Serial.print(F("val2 = "));
  Serial.println(val2);
  Serial.print(F("val3 = "));
  Serial.println(val3);  
  Serial.print(F("val4 = "));
  Serial.println(val4); 
  Serial.print(F("type = "));
  Serial.println(alar.type); 
  Serial.print(F("set = "));
  Serial.println(alar.is_set);   
  Serial.print(F("tripped = "));
  Serial.println(alar.is_tripped);    
#endif 
  
  switch (alarm_number)
  {
     case 1: // write alarm 1
       EEPROM.write(A1A,val1);
       EEPROM.write(A1A+1,val2);
       EEPROM.write(A1A+2,val3);
       EEPROM.write(A1A+3,val4);
       EEPROM.write(A1A+4,alar.type);
       EEPROM.write(A1A+5,alar.is_set);
       EEPROM.write(A1A+6,alar.is_tripped);     
       break; 
     case 2: // write alarm 2
       EEPROM.write(A2A,val1);
       EEPROM.write(A2A+1,val2);
       EEPROM.write(A2A+2,val3);
       EEPROM.write(A2A+3,val4);
       EEPROM.write(A2A+4,alar.type);
       EEPROM.write(A2A+5,alar.is_set);
       EEPROM.write(A2A+6,alar.is_tripped); 
       break;   
     case 3: // write alarm 3
       EEPROM.write(A3A,val1);
       EEPROM.write(A3A+1,val2);
       EEPROM.write(A3A+2,val3);
       EEPROM.write(A3A+3,val4);
       EEPROM.write(A3A+4,alar.type);
       EEPROM.write(A3A+5,alar.is_set);
       EEPROM.write(A3A+6,alar.is_tripped);
       break; 
     case 4: // write alarm 4
       EEPROM.write(A4A,val1);
       EEPROM.write(A4A+1,val2);
       EEPROM.write(A4A+2,val3);
       EEPROM.write(A4A+3,val4);
       EEPROM.write(A4A+4,alar.type);
       EEPROM.write(A4A+5,alar.is_set);
       EEPROM.write(A4A+6,alar.is_tripped);      
       break;      
     case 5: // write alarm 5
       EEPROM.write(A5A,val1);
       EEPROM.write(A5A+1,val2);
       EEPROM.write(A5A+2,val3);
       EEPROM.write(A5A+3,val4);
       EEPROM.write(A5A+4,alar.type);
       EEPROM.write(A5A+5,alar.is_set);
       EEPROM.write(A5A+6,alar.is_tripped);       
       break;   
     default:
       Serial.print(F("ERROR: write_alarm() bad alarm_number = "));
       Serial.println(alarm_number);
       break; 
  }       
}

void readAlarm(int alarm_number){
#if defined(DEBUG)   
  Serial.println(F("read_alarm")); 
#endif  
  uint8_t temp;
  switch (alarm_number)
  {
     case 1: // read alarm 1
       alarm1.datetime = (uint32_t)EEPROM.read(A1A) + \
                               ((uint32_t)(EEPROM.read(A1A+1))<<8) + \
                               ((uint32_t)(EEPROM.read(A1A+2))<<16) + \
                               ((uint32_t)(EEPROM.read(A1A+3))<<24); 
       alarm1.type = EEPROM.read(A1A+4);
       temp = EEPROM.read(A1A+5); // To prevent unprogrammed EEPROM from setting to not 0
       if( temp == 255) alarm1.is_set = 0;
       else alarm1.is_set = temp;
       alarm1.is_tripped = EEPROM.read(A1A+6);     
       #if defined(DEBUG)   
         Serial.println(F("read_alarm1")); 
         Serial.print(F("alarm1.datetime = "));
         Serial.println(alarm1.datetime.unixtime());
         Serial.print(F("alarm1.type = "));
         Serial.println(alarm1.type);
         Serial.print(F("alarm1.is_set = "));
         Serial.println(alarm1.is_set);
         Serial.print(F("alarm1.is_tripped = "));
         Serial.println(alarm1.is_tripped);
       #endif 
       break; 
     case 2: // read alarm 2
       alarm2.datetime = (uint32_t)EEPROM.read(A2A) + \
                               ((uint32_t)EEPROM.read(A2A+1)<<8) + \
                               ((uint32_t)EEPROM.read(A2A+2)<<16) + \
                               ((uint32_t)EEPROM.read(A2A+3)<<24); 
       alarm2.type = EEPROM.read(A2A+4);
       //alarm2.is_set = EEPROM.read(A2A+5);
       temp = EEPROM.read(A2A+5); // To prevent unprogrammed EEPROM from setting to not 0
       if( temp == 255) alarm2.is_set = 0;
       else alarm2.is_set = temp;      
       alarm2.is_tripped = EEPROM.read(A2A+6);        
       break; 
      case 3: // read alarm 3
       alarm3.datetime = (uint32_t)EEPROM.read(A3A) + \
                               ((uint32_t)EEPROM.read(A3A+1)<<8) + \
                               ((uint32_t)EEPROM.read(A3A+2)<<16) + \
                               ((uint32_t)EEPROM.read(A3A+3)<<24); 
       alarm3.type = EEPROM.read(A3A+4);
       temp = EEPROM.read(A3A+5); // To prevent unprogrammed EEPROM from setting to not 0
       if( temp == 255) alarm3.is_set = 0;
       else alarm3.is_set = temp;
       //alarm3.is_set = EEPROM.read(A3A+5);
       alarm3.is_tripped = EEPROM.read(A3A+6);     
       break;    
      case 4: // read alarm 4
       alarm4.datetime = (uint32_t)EEPROM.read(A4A) + \
                               ((uint32_t)EEPROM.read(A4A+1)<<8) + \
                               ((uint32_t)EEPROM.read(A4A+2)<<16) + \
                               ((uint32_t)EEPROM.read(A4A+3)<<24); 
       alarm4.type = EEPROM.read(A4A+4);
       //alarm4.is_set = EEPROM.read(A4A+5);
       temp = EEPROM.read(A4A+5); // To prevent unprogrammed EEPROM from setting to not 0
       if( temp == 255) alarm4.is_set = 0;
       else alarm4.is_set = temp;
       alarm4.is_tripped = EEPROM.read(A4A+6);  
       break;    
      case 5: // read alarm 5
       alarm5.datetime = (uint32_t)EEPROM.read(A5A) + \
                               ((uint32_t)EEPROM.read(A5A+1)<<8) + \
                               ((uint32_t)EEPROM.read(A5A+2)<<16) + \
                               ((uint32_t)EEPROM.read(A5A+3)<<24); 
       alarm5.type = EEPROM.read(A5A+4);
       temp = EEPROM.read(A5A+5); // To prevent unprogrammed EEPROM from setting to not 0
       if( temp == 255) alarm5.is_set = 0;
       else alarm5.is_set = temp;
       //alarm5.is_set = EEPROM.read(A5A+5);
       alarm5.is_tripped = EEPROM.read(A5A+6);  
       break;      
     default:
       Serial.println("ERROR: read_alarm() bad alarm_number ");
  }   
}



